(ns com.aurellem.gb.types
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def type-code->type-name
  {0x00 :normal
   0x01 :fighting
   0x02 :flying
   0x03 :poision
   0x04 :ground
   0x05 :rock
   0x07 :bug
   0x08 :ghost
   0x14 :fire
   0x15 :water
   0x16 :grass
   0x17 :electric
   0x18 :psychic
   0x19 :ice
   0x1A :dragon    })

(def type-name->type-code
  (zipmap (vals type-code->type-name)
          (keys type-code->type-name)))

(def pokemon-1-type-start-address 0xD16F)

(defn pokemon-type-start-address [poke-num]
  (+ pokemon-1-type-start-address
     (* pokemon-record-width poke-num)))

(defn give-type
  ([^SaveState state poke-num types]
     (assert (<= 0 poke-num 5))
     (let [types*
           (if (= (count types) 1)
             [(first types) (first types)]
             types)]
       (set-memory-range
        state
        (pokemon-type-start-address poke-num)
        (map type-name->type-code types))))
  ([poke-num types]
     (give-type @current-state poke-num types)))

(defn read-type
  ([^SaveState state poke-num]
     (assert (<= 0 poke-num 5))
     (let [types-start (pokemon-type-start-address poke-num)
           [type-1 type-2]
           (subvec (vec (memory state))
                   types-start (+ 2 types-start))]
       (if (= type-1 type-2)
         [(type-code->type-name type-1)]
         (mapv type-code->type-name [type-1 type-2]))))
  ([poke-num]
     (read-type @current-state poke-num)))



:normal   nor
:fighting fig
:flying   fly
:poision  poi
:ground   gro
:rock     roc
:bug      bug
:ghost    gho
:fire     fir
:water    wat
:grass    gra
:electric ele
:psychic  psy
:ice      ice
:dragon   dra



(def pokemon->type
  {
   :abra            psy
   :aerodactyl      rck fly
   :alakazam        psy
   :arbok           poi
   :arcanine        fir
   :articuno        ice fly
   :beedrill        bug poi
   :bellsprout      gra poi
   :blastoise       wat
   :bulbasaur       gra poi
   :butterfree      bug fly
   :caterpie        bug
   :chansey         nor
   :charizard       fir fly
   :charmander      fir
   :charmeleon      fir
   :clefable        nor
   :clefairy        nor
   :cloyster        wat ice
   :cubone          gro
   :dewgong         wat ice
   :diglett         gro
   :ditto           nor
   :dodrio          nor fly
   :doduo           nor fly
   :dragonair       drg
   :dragonite       drg fly
   :dratini         drg
   :drowzee         psy
   :dugtrio         gro
   :eevee           nor
   :ekans           poi
   :electabuzz      ele
   :electrode       ele
   :exeggcute       gra psy
   :exeggutor       gra psy
   :farfetchd       nor fly
   :fearow          nor fly
   :flareon         fir
   :gastly          gho poi
   :gengar          gho poi
   :geodude         rck gro
   :gloom           gra poi
   :golbat          poi fly
   :goldeen         wat
   :golduck         wat
   :golem           rck gro
   :graveler        rck gro
   :grimer          poi
   :growlithe       fir
   :gyarados        wat fly
   :haunter         gho poi
   :hitmonchan      fgt
   :hitmonlee       fgt
   :horsea          wat
   :hypno           psy
   :ivysaur         gra poi
   :jigglypuff      nor
   :jolteon         ele
   :jynx            ice psy
   :kabuto          rck wat
   :kabutops        rck wat
   :kadabra         psy
   :kakuna          bug poi
   :kangaskhan      nor
   :kingler         wat
   :koffing         poi
   :krabby          wat
   :lapras          wat ice
   :lickitung       nor
   :machamp         fgt
   :machoke         fgt
   :machop          fgt
   :magikarp        wat
   :magmar          fir
   :magnemite       ele
   :magneton        ele
   :mankey          fgt
   :marowak         gro
   :meowth          nor
   :metapod         bug
   :mew             psy
   :mewtwo          psy
   :moltres         fir fly
   :mr-mime         psy
   :muk             poi
   :nidoking        poi gro
   :nidoqueen       poi gro
   :nidoran-female  poi
   :nidoran-male    poi
   :nidorina        poi
   :nidorino        poi
   :ninetails       fir
   :oddish          gra poi
   :omanyte         rck wat
   :omastar         rck wat
   :onix            rck gro
   :paras           bug gra
   :parasect        bug gra
   :persian         nor
   :pidgeot         nor fly
   :pidgeotto       nor fly
   :pidgey          nor fly
   :pikachu         ele
   :pinsir          bug
   :poliwag         wat
   :poliwhirl       wat
   :poliwrath       wat fgt
   :ponyta          fir
   :porygon         nor
   :primeape        fgt
   :psyduck         wat
   :raichu          ele
   :rapidash        fir
   :raticate        nor
   :rattata         nor
   :rhydon          gro rck
   :rhyhorn         gro rck
   :sandshrew       gro
   :sandslash       gro
   :scyther         bug fly
   :seaking         wat
   :sedra           wat
   :seel            wat
   :shellder        wat
   :slowbro         wat psy
   :slowpoke        wat psy
   :snorlax         nor
   :spearow         nor fly
   :squirtle        wat
   :starmie         wat psy
   :staryu          wat
   :tangela         gra
   :tauros          nor
   :tentacool       wat poi
   :tentacruel      wat poi
   :vaporeon        wat
   :venomoth        bug poi
   :venonat         bug poi
   :venusaur        gra poi
   :victreebel      gra poi
   :vileplume       gra poi
   :voltorb         ele
   :vulpix          fir
   :wartortle       wat
   :weedle          bug poi
   :weepenbell      gra poi
   :wheezing        poi
   :wigglytuff      nor
   :zapdos          ele fly
   :zubat           poi fly